/*
	A minimal Ox implementation of a dictionary (associative array).

	Syntax:
	decl dict = new Dict( { "key1", val1, "key2", val2, ..., "keyN", valN } );
	if(dict.hasKey("key3")) {
		println(dict.get("key3"));
	}
	dict.set("newkey", newval);
	delete dict;
*/

class Dict {
	decl keys, values;
	Dict(const input);
	hasKey(const key);
	get(const key);
	set(const key, const value);
	allKeys();
	allValues();
	print();
}

Dict::Dict(const input) {

	decl inputsize = sizeof(input);

	if(imod(inputsize, 2) != 0) {
		oxrunerror("Dict: constructor array has odd number of elements (" ~ sprint(inputsize) ~ ")", 0);
	}

	decl pairs = inputsize / 2;

	keys = new array[pairs];
	values = new array[pairs];
	
	for(decl i = 0; i < pairs; i++) {
		if(!isstring(input[2*i])) {
			oxrunerror("Dict: key " ~ sprint(i+1) ~ " is not a string", 0);
		}
		keys[i] = input[2*i];
		values[i] = input[2*i + 1];
	}
}

Dict::hasKey(const key) {
	return (strfind(keys, key) != -1);
}

Dict::get(const key) {
	decl index = strfind(keys, key);
	if(index != -1) {
		return values[index];
	} else {
		oxrunerror("Dict: key '" ~ key ~ "' was not found", 0);
	}
}

Dict::set(const key, const value) {
	decl index = strfind(key, value);
	if(index != -1) {
		values[index] = value;
	} else {
		keys ~= key;
		values ~= value;
	}
}

Dict::allKeys() {
	return keys;
}

Dict::allValues() {
	return values;
}

Dict::print() {
	for(decl i = 0; i < sizeof(keys); i++) {
		println("'" ~ keys[i] ~ "': " ~ sprint(values[i]));
	}
}