#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

#include "Misc.ox"

/*
	This script computes the mixture of two normals that best approximates a standardized chi-squared distribution.
*/

// given the weight and parameters for one normal, the parameters for the other one follow from the standardization restrictions
getOtherParametersForMixture(const weight1, const mu1, const sigma1) {
	decl weight2 = 1 - weight1;
	decl mu2 = (-mu1 * weight1) / weight2;
	decl sigma2 = sqrt((1 - weight1*(mu1^2 + sigma1^2))/weight2 - mu2^2);
	return {weight2, mu2, sigma2};
}

main() {
	decl samples = ranchi(1, 1000000, 6);
	// standardize to zero mean and unit variance
	samples -= meanr(samples);
	samples /= sqrt(varr(samples));

	decl par = <0.5, 0.5, 0.5>;
	par = Transform(par, <2, 1, 1>)';
	
	decl loglik = [=] (const par, const aval, const ascore, ahes) {
		decl truepar = Transform(par, <-2, -1, -1>)';
		decl weight1 = truepar[0], mu1 = truepar[1], sigma1 = truepar[2];
		decl weight2, mu2, sigma2; [weight2, mu2, sigma2] = getOtherParametersForMixture(weight1, mu1, sigma1);
		aval[0] = sumr(log(
			(weight1 * densn((samples - mu1) / sigma1) / sigma1)
				+
			(weight2 * densn((samples - mu2) / sigma2) / sigma2)
		));
		return 1;
	};

	decl val;
	MaxBFGS(loglik, &par, &val, 0, 1);
	par = Transform(par, <-2, -1, -1>)';

	decl weight1 = par[0], mu1 = par[1], sigma1 = par[2];
	decl weight2, mu2, sigma2; [weight2, mu2, sigma2] = getOtherParametersForMixture(weight1, mu1, sigma1);
	println(weight1~mu1~sigma1~weight2~mu2~sigma2);
	DrawDensity(0, samples, 0);
	ShowDrawWindow();
}