/*
	The GAS DGPs.

	Common arguments:
		T: time series length
		mu: mean
		omega, alpha, beta: GAS parameters

	Returns an array:
		[0]: resulting data
		[1]: underlying f_t
*/
GASgenerator_Normal(const T, const mu, const omega, const alpha, const beta) {
	decl eps_t = rann(1, T), f_t = zeros(1, T), y_t = zeros(1, T);
	f_t[0] = omega;
	decl i = 0;
	while (TRUE) {
		y_t[i] = mu + exp(f_t[i]/2) * eps_t[i];
		if(i == T-1) break;
		f_t[i+1] = omega * (1 - beta) + alpha * (-1/2) * (eps_t[i] * dlogn(eps_t[i]) + 1) + beta * f_t[i];
		++i;
	}
	return {y_t, f_t};
}

// extra argument: nu
GASgenerator_t(const T, const mu, const omega, const alpha, const beta, const nu) {
	decl eps_t = rannormt(1, T, nu), f_t = zeros(1, T), y_t = zeros(1, T);
	f_t[0] = omega;
	decl i = 0;
	while (TRUE) {
		y_t[i] = mu + exp(f_t[i]/2) * eps_t[i];
		if(i == T-1) break;
		f_t[i+1] = omega * (1 - beta) + alpha * (-1/2) * (eps_t[i] * dlognormt(eps_t[i], nu) + 1) + beta * f_t[i];
		++i;
	}
	return {y_t, f_t};
}

GASgenerator_MN(const T, const mu, const omega, const alpha, const beta) {
	decl eps_t = ranchimix(1, T), f_t = zeros(1, T), y_t = zeros(1, T);
	f_t[0] = omega;
	decl i = 0;
	while (TRUE) {
		y_t[i] = mu + exp(f_t[i]/2) * eps_t[i];
		if(i == T-1) break;
		f_t[i+1] = omega * (1 - beta) + alpha * (-1/2) * (eps_t[i] * dlogchimix(eps_t[i]) + 1) + beta * f_t[i];
		++i;
	}
	return {y_t, f_t};
}

/*
	Non-GAS generators.

	Arguments:
		T: time series length

	Returns a 1xT vector of volatility (sigma)
*/

Vol_Sine(const T) {
	return 2.5 + cos(2*M_PI*range(0, T-1)/200);
}

Vol_LongRegimes(const T) {
	return 1.5 + 2 * (1 - imod(range(0, T-1)/500, 2));
}

Vol_ShortRegimes(const T) {
	return 1.5 + 2 * (1 - imod(range(0, T-1)/200, 2));
}

Vol_Bursts(const T) {
	decl result = <>;
	decl onecycle = 1.5 * ones(1, 150) ~ range(1.9, 3.5, 0.4) ~ range(3.5-2/95, 1.5, -2/95);
	decl bursts = idiv(T, 250);
	for(decl i = 0; i < bursts; i++) {
		result ~= onecycle;
	}
	result ~= 1.5 * ones(1, T - bursts * 250);
	return result;
}

Vol_Stoch(const T) {
	decl alpha = 0.5;
	decl eta = 0.1 * rann(1, T);
	decl result = zeros(1, T);
	for(decl i = 0; i < T; i++) {
		alpha =	0.01 + 0.98 * alpha + eta[i];
		result[i] = exp(alpha/2);
	}
	return result;
}