/*
	The kernel estimation class.

	A kernel density estimate is constructed with the given points and bandwidth (and a t kernel).
	It is automatically normalized to zero mean and unit variance.
*/
static decl KERNEL_T_NU = 5;

class StandardizedKernelEstimator {
	decl points, bandwidth;
	StandardizedKernelEstimator(const points, const bandwidth);
	dens(const x);
	ddens(const x);
	ddens2(const x);
	dlog(const x);
};

/*
	Construct the standardized kernel estimate.

	points: 1xT vector of data points
	bandwidth: pre-normalization bandwidth
*/
StandardizedKernelEstimator::StandardizedKernelEstimator(const points, const bandwidth) {
	decl mixturemean = meanr(points);
	decl mixturesd = sqrt(meanr((points-mixturemean).^2 + bandwidth^2));
	this.points = (points - mixturemean) / mixturesd;
	this.bandwidth = bandwidth / mixturesd;
}
/*
	Compute the density estimate.

	x: 1xn vector of values at which to evaluate the density
*/
StandardizedKernelEstimator::dens(const x) {
	return meanr(densnormt((x' - points) / bandwidth, KERNEL_T_NU) / bandwidth)';
}

/*
	Compute the derivative of the density.

	x: 1xn vector values at which to evaluate the derivative
*/
StandardizedKernelEstimator::ddens(const x) {
	return meanr(-densnormt((x' - points) / bandwidth, KERNEL_T_NU) .* (x' - points) * (KERNEL_T_NU + 1) ./ (sqr(x' - points) + bandwidth^2 * (KERNEL_T_NU - 2)) / bandwidth)';
}

/*
	Compute the derivative of the logarithm of the density (used for SP-GAS).

	x: 1xn vector values at which to evaluate the derivative
*/
StandardizedKernelEstimator::dlog(const x) {
	return ddens(x) ./ dens(x);
}