#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

#include "Misc.ox"
#include "Dictionary.ox"
#include "Distributions.ox"
#include "StandardizedKernelEstimatorT.ox"
#include "Estimators.ox"
#include "Generators.ox"

main() {
	decl T = 2000;
	decl replications = 100;
	decl K = 1000; // how much data to use for estimation

	decl mu = 0, omega = 2, alpha = 0.3, beta = 0.9;
	decl truepar = mu ~ omega ~ alpha ~ beta;
	
	// read command line number
	decl tablepart; sscan(arglist()[1], &tablepart);

	decl progress = 0;

	// 31 columns:
	// MLE mu (error), MLE omega, MLE alpha, MLE beta, MLE in (RMSE), MLE out,
	// QML mu, ... SP(AIC) out, SP iterations
	decl rundata = zeros(replications, 31);

	while(progress < replications) {
		decl data, truef_t, MLresults, GASresults;

		switch_single(tablepart) {
		 	case 1:
				[data, truef_t] = GASgenerator_Normal(T, mu, omega, alpha, beta);
				MLresults = GAS_Normal(data, K, truepar);
			case 2:
				[data, truef_t] = GASgenerator_t(T, mu, omega, alpha, beta, 3);
				MLresults = GAS_t3(data, K, truepar);
			case 3:
				[data, truef_t] = GASgenerator_t(T, mu, omega, alpha, beta, 5);
				MLresults = GAS_t5(data, K, truepar);
			case 4:
				[data, truef_t] = GASgenerator_MN(T, mu, omega, alpha, beta);
				MLresults = GAS_MN(data, K, truepar);
			default:
				oxrunerror("Table part number is invalid", 0);
		}

		if(!MLresults.get("succes")) {
			delete MLresults;
			continue;
		}

		GASresults = GAS(data, K, truepar ~ (5 - 2*(tablepart==2)));

		if(!GASresults.get("succes")) {
			delete MLresults, GASresults;
			continue;
		}

		// store data
		rundata[progress][0:3] = MLresults.get("par") - truepar;
		rundata[progress][6:9] = GASresults.get("QML-par") - truepar;
		rundata[progress][12:15] = GASresults.get("t-par")[0:3] - truepar;
		rundata[progress][18:21] = GASresults.get("SP1-par") - truepar;
		rundata[progress][24:27] = GASresults.get("SPX-par") - truepar;

		decl f_t = {MLresults.get("ft"), GASresults.get("QML-ft"),
					GASresults.get("t-ft"), GASresults.get("SP1-ft"),
					GASresults.get("SPX-ft")};

		for(decl i = 0; i < 5; i++) {
			rundata[progress][4 + 6*i] = RMSE(exp(f_t[i][:K-1] / 2), exp(truef_t[:K-1] / 2));
			rundata[progress][5 + 6*i] = RMSE(exp(f_t[i][K:] / 2), exp(truef_t[K:] / 2));
		}
		
		rundata[progress][30] = GASresults.get("SPiter");

		progress++;
		
		delete MLresults, GASresults;
	}

	savemat("Table1-" ~ sprint(tablepart) ~ ".mat", rundata);
	
}