#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

#include "Misc.ox"
#include "Dictionary.ox"
#include "Distributions.ox"
#include "StandardizedKernelEstimatorT.ox"
#include "Estimators.ox"
#include "Generators.ox"

main() {
	decl T = 2000;
	decl replications = 100;
	decl K = 1000; // how much data to use for estimation

	// now these are just initial parameters
	decl mu = 0, omega = 1, alpha = 0.1, beta = 0.9;
	decl initpar = mu ~ omega ~ alpha ~ beta;
	
	// read command line number
	decl tablepart; sscan(arglist()[1], &tablepart);

	decl volprocess = imod(tablepart - 1, 4);
	decl distribution = idiv(tablepart - 1, 4);

	decl progress = 0;

	// 11 columns:
	// MLE in (RMSE), MLE out, QML in, QML out,
	// t in, ... SP(AIC) out, SP iterations
	decl rundata = zeros(replications, 11);

	while(progress < replications) {
		decl data, truevol, MLresults, GASresults;

		switch_single(volprocess) {
		 	case 0:
				truevol = Vol_Sine(T);
			case 1:
				truevol = Vol_LongRegimes(T);
			case 2:
				truevol = Vol_ShortRegimes(T);
			case 3:
				truevol = Vol_Bursts(T);
		}

		switch_single(distribution) {
			case 0:
				data = truevol .* rann(1, T);
				MLresults = GAS_Normal(data, K, initpar);
			case 1:
				data = truevol .* rannormt(1, T, 3);
				MLresults = GAS_t3(data, K, initpar);
			case 2:
				data = truevol .* rannormt(1, T, 5);
				MLresults = GAS_t5(data, K, initpar);
			case 3:
				data = truevol .* ranchimix(1, T);
				MLresults = GAS_MN(data, K, initpar);
			default:
				oxrunerror("Table part number is invalid", 0);
		}

		if(!MLresults.get("succes")) {
			delete MLresults;
			continue;
		}

		GASresults = GAS(data, K, initpar ~ (5 - 2*(tablepart==2)));

		if(!GASresults.get("succes")) {
			delete MLresults, GASresults;
			continue;
		}

		// store data
		decl f_t = {MLresults.get("ft"), GASresults.get("QML-ft"),
					GASresults.get("t-ft"), GASresults.get("SP1-ft"),
					GASresults.get("SPX-ft")};

		for(decl i = 0; i < 5; i++) {
			rundata[progress][0 + 2*i] = RMSE(exp(f_t[i][:K-1] / 2), truevol[:K-1]);
			rundata[progress][1 + 2*i] = RMSE(exp(f_t[i][K:] / 2), truevol[K:]);
		}
		
		rundata[progress][10] = GASresults.get("SPiter");

		progress++;
		
		delete MLresults, GASresults;
	}

	savemat("Table2-" ~ sprint(distribution+1) ~ "-" ~ sprint(volprocess+1) ~ ".mat", rundata);
	
}