#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

#include "Misc.ox"
#include "Dictionary.ox"
#include "Distributions.ox"
#include "StandardizedKernelEstimatorT.ox"
#include "Estimators.ox"
#include "Generators.ox"

main() {
	decl T = 2000;
	decl replications = 100;
	decl K = 1000; // how much data to use for estimation

	// initial parameters
	decl GASinitpar = <0, -0.5, 0.1, 0.8>;
	decl GARCHinitpar = <0, 1.5, 0.1, 0.8>; 
	
	// read command line number
	decl tablepart; sscan(arglist()[1], &tablepart);

	decl progress = 0;

	// 18 columns:
	// GAS QML in, GAS QML out, GAS t in, ...
	// ... GAS SP(AIC) out, GAS SP iterations, GARCH QML in, GARCH QML out, ...
	// ... GARCH SP(AIC) out, GARCH SP iterations
	decl rundata = zeros(replications, 18);

	while(progress < replications) {
		println(progress);
		decl data, GASresults, GARCHresults;

		decl truevol = Vol_Stoch(T);

		switch_single(tablepart) {
			case 1:
				data = truevol .* rann(1, T);
			case 2:
				data = truevol .* rannormt(1, T, 3);
			case 3:
				data = truevol .* rannormt(1, T, 5);
			case 4:
				data = truevol .* ranchimix(1, T);
			default:
				oxrunerror("Table part number is invalid", 0);
		}

		GASresults = GAS(data, K, GASinitpar ~ (5 - 2*(tablepart==2)));

		if(!GASresults.get("succes")) {
			delete GASresults;
			continue;
		}

		print("G");
		GARCHresults = GARCH(data, K, GARCHinitpar ~ (5 - 2*(tablepart==2)));

		if(!GARCHresults.get("succes")) {
			delete GASresults, GARCHresults;
			continue;
		}

		// store data
		decl f_t = {GASresults.get("QML-ft"), GASresults.get("t-ft"),
					GASresults.get("SP1-ft"), GASresults.get("SPX-ft")};

		decl sigma2_t = {GARCHresults.get("QML-sigma2t"), GARCHresults.get("t-sigma2t"),
						GARCHresults.get("SP1-sigma2t"), GARCHresults.get("SPX-sigma2t")};

		for(decl i = 0; i < 4; i++) {
			rundata[progress][0 + 2*i] = RMSE(exp(f_t[i][:K-1] / 2), truevol[:K-1]);
			rundata[progress][1 + 2*i] = RMSE(exp(f_t[i][K:] / 2), truevol[K:]);
		}

		for(decl i = 0; i < 4; i++) {
			rundata[progress][9 + 2*i] = RMSE(sqrt(sigma2_t[i][:K-1]), truevol[:K-1]);
			rundata[progress][10 + 2*i] = RMSE(sqrt(sigma2_t[i][K:]), truevol[K:]);
		}
						
		rundata[progress][8] = GASresults.get("SPiter");
		rundata[progress][17] = GARCHresults.get("SPiter");

		progress++;
		
		delete GASresults, GARCHresults;
	}

	savemat("Table3-" ~ sprint(tablepart) ~ ".mat", rundata);
	
}